#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue May 31 16:53:42 2018

Developed and tested in : 
- Python version 3.6
- Linux Ubuntu version 16.04 LTS (64-bit)

@author: Nikola Knezevic
"""

import requests
import time
from collections import OrderedDict
import json
import os
import sys


####################################################################################################
########################################### PARAMETERS #############################################

# url of WISeREP2.0 API
url_wis_api="https://wiserep.weizmann.ac.il/api"

# Specify here the relevant API key
api_key="1234567890123456789012345678901234567890"

# API key json keyword (if API key is personal or bot's API key)
# It can have following values:
# - "personal_api_key"
# - "bot_api_key"
#api_key_json_keyword="personal_api_key" 
api_key_json_keyword="bot_api_key" 

# sleeping time (in sec)
TIME_SLEEP=2

# max number of time to check response
LOOP_COUNTER=60 

# http errors
http_errors = {                                                                               
304: 'Error 304: Not Modified: There was no new data to return.',
400: 'Error 400: Bad Request: The request was invalid. '\
     'An accompanying error message will explain why.',
403: 'Error 403: Forbidden: The request is understood, but it has '\
     'been refused. An accompanying error message will explain why.',
404: 'Error 404: Not Found: The URI requested is invalid or the '\
     'resource requested, such as a category, does not exists.',
500: 'Error 500: Internal Server Error: Something is broken.',
503: 'Error 503: Service Unavailable.'
}

# keeping sys.stdout
old_stdout = sys.stdout

####################################################################################################
####################################################################################################



####################################################################################################
########################################### FUNCTIONS ##############################################

# function for uploading files trough api
def upload_files(url,list_of_files,list_of_files_path):
  try:
    # url for uploading files
    upload_url=url+'/file-upload'
    # construct the list of (key,value) pairs
    files=[(api_key_json_keyword,(None, api_key))]
    # go trough every file in the list_of_files and add that
    # to the list of (key,value) pairs for uploading
    for i in range(len(list_of_files)):
        file_name=list_of_files[i]
        file_path=list_of_files_path[i]
        key='files['+str(i)+']'
        value=(file_name,open(file_path,'rb'))
        files.append((key,value))
    # upload all files using request module
    response=requests.post(upload_url, files=files)
    # return response
    return response
  except Exception as e:
    return [None,'Error message : \n'+str(e)]

# function that checks response and
# returns True if everything went OK
# or returns False if something went wrong
def check_response(response):
    # if response exists
    if None not in response:
        # take status code of that response
        status_code=int(response.status_code)
        if status_code==200:          
            # response as json data
            json_data=response.json()
            # id code
            id_code=str(json_data['id_code'])
            # id message
            id_message=str(json_data['id_message'])
            # print id code and id message
            print ("ID code = "+id_code)
            print ("ID message = "+id_message)
            # check if id code is 200 and id message OK
            if (id_code=="200" and id_message=="OK"):
                return True
            #special case
            elif (id_code=="400" and id_message=="Bad request"):
                return None
            else:
                return False       
        else:
            # if status code is not 200, check if it exists in
            # http errors
            if status_code in list(http_errors.keys()):
                print (list(http_errors.values())
                       [list(http_errors.keys()).index(status_code)])
            else:
                print ("Undocumented error.")
            return False
    else:
        # response doesn't exists, print error
        print (response[1])
        return False

# uploading files and printing reply
def upload(url,list_of_files,list_of_files_path):
    # upload files and checking response
    print ("Uploading files on the WISeREP...\n")
    time.sleep(TIME_SLEEP)
    response=upload_files(url,list_of_files,list_of_files_path)
    response_check=check_response(response)
    time.sleep(TIME_SLEEP)
    # if files are uploaded
    if response_check==True:
        print ("\nThe following files are uploaded on the WISeREP : \n")
        time.sleep(TIME_SLEEP)
        # response as json data
        json_data=response.json()
        # list of uploaded files
        uploaded_files=json_data['data']
        for i in range(len(uploaded_files)):
            print ("-filename : "+str(uploaded_files[i]))
        print("\n")
        time.sleep(TIME_SLEEP)
        return True
    else:
        print ("\nFiles are not uploaded on the WISeREP.\n")
        time.sleep(TIME_SLEEP)
        return False

# function for changing data to json format
def format_to_json(source):
    # change data to json format and return
    parsed=json.loads(source,object_pairs_hook=OrderedDict)
    result=json.dumps(parsed,indent=4)
    return result

# function for sending TSV metadata
def send_tsv(url,tsv_metadata,tsv_metadata_path):
  try:
    # url for sending tsv metadata
    tsv_url=url+'/csv-report'
    # tsv file path
    tsv_file_path=tsv_metadata_path
    # construct list of (key,value) pairs
    tsv_data=[(api_key_json_keyword,(None, api_key)),
              ('csv',(tsv_metadata,open(tsv_file_path,'rb')))]
    # send tsv metadata using request module
    response=requests.post(tsv_url, files=tsv_data)
    # return response
    return response
  except Exception as e:
    return [None,'Error message : \n'+str(e)]

# function for sending json metadata
def send_json(url,json_metadata,json_metadata_path):
  try:
    # url for sending json metadata
    json_url=url+'/bulk-report'
    # json file path
    json_file_path=json_metadata_path
    # read json data from file
    json_read=format_to_json(open(json_file_path).read())
    # construct list of (key,value) pairs
    json_data=[(api_key_json_keyword,(None, api_key)),
               ('data',(None,json_read))]
    # send json metadata using request module
    response=requests.post(json_url, files=json_data)
    # return response
    return response
  except Exception as e:
    return [None,'Error message : \n'+str(e)]

# Disable print
def blockPrint():
    sys.stdout = open(os.devnull,'w')

# Restore print
def enablePrint():
    sys.stdout.close()
    sys.stdout = old_stdout

# function for getting reply from report
def reply(url,report_id):
  try:
    # url for getting report reply
    reply_url=url+'/bulk-report-reply'
    # construct list of (key,value) pairs  
    reply_data=[(api_key_json_keyword,(None, api_key)),
                ('report_id',(None,report_id))]
    # send report ID using request module
    response=requests.post(reply_url, files=reply_data)
    # return response
    return response
  except Exception as e:
    return [None,'Error message : \n'+str(e)]

# sending report id to get reply of the report
# and printing that reply
def print_reply(url,report_id):
    # sending reply using report id and checking response
    print ("Sending reply for the report ID "+report_id+" ...\n")
    time.sleep(TIME_SLEEP)
    reply_res=reply(url, report_id)
    reply_res_check=check_response(reply_res)
    time.sleep(TIME_SLEEP)
    # if reply is sent
    if reply_res_check==True:
        print ("\nThe report was successfully processed on the WISeREP.\n")
        time.sleep(TIME_SLEEP)
        # reply response as json data
        json_data=reply_res.json()
        # print feedback of the response
        print('"feedback":')
        print(json.dumps(json_data['data']['feedback'],indent=4))
        print("\n")
        time.sleep(TIME_SLEEP)
        return True
    else:
        if (reply_res_check!=None):
            print ("\nThe report doesn't exist on the WISeREP.\n")
            time.sleep(TIME_SLEEP)
        else:
            print ("\nThe report was not processed on the WISeREP "\
                   "because of the bad request(s).\n")
            # reply response as json data
            json_data=reply_res.json()
            print('"feedback":')
            print(json.dumps(json_data['data']['feedback'],indent=4))
            print("\n")
            time.sleep(TIME_SLEEP)
        return False

# sending tsv or json metadata file and printing reply
def send_metadata(url,metadata,metadata_path,type_of_metadata):
    # sending metadata and checking response
    print ("Sending "+metadata+" metadata to the WISeREP...\n")
    time.sleep(TIME_SLEEP)
    # choose which function to call
    if type_of_metadata=="tsv":
        response=send_tsv(url,metadata,metadata_path)
    else:
        response=send_json(url,metadata,metadata_path)
    response_check=check_response(response)
    time.sleep(TIME_SLEEP)
    # if metadata is sent
    if response_check==True:
        print ("\nThe metadata was sent to the WISeREP.\n")
        time.sleep(TIME_SLEEP)
        # report response as json data
        json_data=response.json()
        # taking report id
        report_id=str(json_data['data']['report_id'])
        print ("Report ID = "+report_id+"\n")
        time.sleep(TIME_SLEEP)
        # sending report id to get reply of the report
        # and printing that reply
        # waiting for report to arrive before sending reply
        # for report id
        blockPrint()
        counter = 0
        while True:
            time.sleep(TIME_SLEEP-1)
            reply_response=reply(url,report_id)
            reply_res_check=check_response(reply_response)
            if reply_res_check!=False or counter >= LOOP_COUNTER:
                break
            counter += 1
        enablePrint()
        print_reply_response=print_reply(url,report_id)
        return print_reply_response
    else:
        print ("\nThe metadata was not sent to the WISeREP.\n")
        time.sleep(TIME_SLEEP)
        return False

####################################################################################################
####################################################################################################



####################################################################################################
############################################ EXAMPLE ###############################################

# current working directory
cwd=os.getcwd()

# folder for files
files_folder=os.path.join(cwd,'files_for_uploading')

# folder for tsv reports
tsvs_folder=os.path.join(cwd,'tsv_reports_for_sending')

# folder for json reports
jsons_folder=os.path.join(cwd,'json_reports_for_sending')

# files for upload
FILES=['example.asci','example.fits','example.png']
FILES_PATH=[os.path.join(files_folder,f) for f in FILES]

# spectra tsv file
TSV_FILE='bulk_tsv_spectra_report.txt'
TSV_FILE_PATH=os.path.join(tsvs_folder,TSV_FILE)

# spectra json file
JSON_FILE='bulk_json_spectra_report.json'
JSON_FILE_PATH=os.path.join(jsons_folder,JSON_FILE)

# photometry tsv file
PHOT_TSV_FILE='bulk_tsv_photometry_report.txt'
PHOT_TSV_FILE_PATH=os.path.join(tsvs_folder,PHOT_TSV_FILE)


"""
#************************ SENDING SPECTRA TSV FILE ******************************

# uploading files
upload_res=upload(url_wis_api,FILES,FILES_PATH)

# sending TSV metadata
if upload_res==True:
    metadata_reply=send_metadata(url_wis_api,TSV_FILE,TSV_FILE_PATH,"tsv")
    if metadata_reply==False:
        print ("Resolve that and run program again.\n")
        time.sleep(TIME_SLEEP)
        # exit the program
        sys.exit()

#********************************************************************************
"""


"""
#*********************** SENDING SPECTRA JSON FILE ******************************

# uploading files
upload_res=upload(url_wis_api,FILES,FILES_PATH)

# sending JSON metadata
if upload_res==True:
    metadata_reply=send_metadata(url_wis_api,JSON_FILE,JSON_FILE_PATH,"json")
    if metadata_reply==False:
        print ("Resolve that and run program again.\n")
        time.sleep(TIME_SLEEP)
        # exit the program
        sys.exit()

#********************************************************************************
"""


"""
#********************** SENDING PHOTOMETRY TSV FILE *****************************

# sending photometry TSV file
metadata_reply=send_metadata(url_wis_api,PHOT_TSV_FILE,PHOT_TSV_FILE_PATH,"tsv")
if metadata_reply==False:
    print ("Resolve that and run program again.\n")
    time.sleep(TIME_SLEEP)
    # exit the program
    sys.exit()

#********************************************************************************
"""


####################################################################################################
####################################################################################################


